##################################################
## Lehrerfortbildung Wil, 7.9.2016              ##
## R-Code zum Workshop: Von Ursache und Wirkung ##
##################################################

## Datenbeispiel aus Erhebung vom Nachmittagsworkshop in Wil
## Der Befehl 'c()' fuer 'combine' erzeugt einen Vektor
## Der Zuweisungspfeil '<-' speichert etwas unter einem Variablennamen ab
gelb <- c(0,1,0,0,1,2,1,0,3,4,0,1,0,0,0,0) ## Anzahl Personen pro Punktzahl 0 bis 15
rot <- c(0,1,0,0,0,0,0,3,4,2,8,0,1,0,0,0)
## Bei einer Zuweisung mit '<-' wird das Ergebnis nicht angezeigt. Wenn man den Inhalt einer Variable
## Anzeigen will, muss man den Variablennamen eingeben und die Zeile ausfuehren:
rot

nGelb <- sum(gelb) ## Berechne Anzahl Teilnehmer in gelber Gruppe
?sum ## Mit dem Fragezeichen kann man die Hilfeseite einer R-Funktion aufrufen
nRot <- sum(rot) ## Berechne Anzahl Teilnehmer in roter Gruppe
gelbL <- rep(0:15,times = gelb) ## 'long version': Alle Punktzahlen pro Person aufgelistet
rotL <- rep(0:15, times = rot)
rotL

########################################
## Beschreibende Statistik
########################################
## Grafiken: barplot
m <- rbind(gelb, rot) ## Mache aus beiden Datenvektoren eine 2*16-Matrix ('rbind' fuer 'row bind')
barplot(m, beside = TRUE, col = c("yellow", "red"))

## Kennzahlen: Mittelwert
mwGelb <- mean(gelbL)
mwRot <- mean(rotL)
mDiff <- mwRot - mwGelb
mDiff 

#########################################
## Schliessende Statistik
#########################################
## Permutationstest
## Anzahl moeglicher Kombinationen nGelb Personen aus nGelb+nRot Personen auszuwaehlen
choose(nGelb+nRot, nGelb) ## Meist zu viele -> Untersuchen nur eine Zufallsauswahl aller moeglichen Permutationen

alle <- c(gelbL, rotL) ## alle Beobachtungen; die ersten nGelb sind Gruppe 'gelb', die restlichen sind Gruppe 'rot'
n <- 10000 ## Anzahl zufaellig ausgewaehlter Permutationen
res <- rep(0,n) ## Bereite einen Vektor vor, in dem die Ergebnisse der n Simulationen gespeichert werden (nicht noetig aber guter Stil)
for (i in 1:n) { ## Wiederhole den Block in den geschweiften Klammern n mal
  permuteAlle <- sample(alle) ## Permutiere Personen 
  mwGelbTmp <- mean(permuteAlle[1:nGelb]) ## berechne neuen Mittelwert in der (neuen) gelben Gruppe nach Permutation
  mwRotTmp <- mean(permuteAlle[(nGelb+1):(nGelb+nRot)]) ## berechne neuen Mittelwert in der (neuen) roten Gruppe nach Permutation
  res[i] <- mwRotTmp - mwGelbTmp ## berechne neue Differenz nach Permutation
} 

hist(res) ## Histogramm der simulierten Differenzen
abline(v=mDiff, col = "red") ## zeichne beobachtete Differenz ein

## p-Wert: Wie wahrscheinlich ist das beobachtete Ergebnis (oder etwas noch extremeres), 
## falls die Nullhypothese (Behandlung hat keinerlei Effekt) stimmt ?
sum(res >= mDiff) / n ## p-Wert stimmt nur approximativ, weil wir nicht alle Permutationen berechnet haben

################################################
## Weiterfuehrende Idee (Funktion programmieren)
################################################
## Wir haben ja 'nur' 10'000 Permutationen verwendet. Wie nah ist unser p-Wert am 'echten' p-Wert 
## (wenn wir also alle Permutationen verwendet haetten)?

## Wiederhole obige Simulation mehrmals mit unterschiedlichen Permuatationen
## Dazu ist es bequem, wenn wir obigen Code-Block in eine FUNKTION verpacken:
runSim <- function(gL, rL, n=10000) { ## n hat default Wert, gelbL und rotL muessen zwingend uebergeben werden
  ## Diese Funktion simuliert den p-Wert eines Simulationstests
  ## Argumente: 
  ## - gL: Num. Vektor mit Punktzahlen in gelber Gruppe
  ## - rL: Num. Vektor mit Punktzahlen in roter Gruppe
  ## - n: Anzahl Permutationen
  ## Ausgabe:
  ## - simulierter p-Wert
  alle <- c(gL, rL) ## alle Beobachtungen; die ersten nGelb sind Gruppe 'gelb', die restlichen sind Gruppe 'rot'
  nGelb <- length(gL)
  nRot <- length(rL)
  mDiff <- mean(rL) - mean(gL) ## Berechne nochmals beobachtete Differenz der Gruppenmittelwerte
  
  ## Simuliere Verteilung der Differenz der Gruppenmittelwerte falls es keinen Behandlungseffekt gibt (d.h., unter der Nullhypothese)
  res <- rep(0,n) ## Bereite einen Vektor vor, in dem die Ergebnisse der n Simulationen gespeichert werden
  for (i in 1:n) { ## Wiederhole den Block in den geschweiften Klammern n mal
    permuteAlle <- sample(alle) ## Permutiere Personen 
    mwGelbTmp <- mean(permuteAlle[1:nGelb]) ## berechne neuen Mittelwert in der (neuen) gelben Gruppe nach Permutation
    mwRotTmp <- mean(permuteAlle[(nGelb+1):(nGelb+nRot)]) ## berechne neuen Mittelwert in der (neuen) roten Gruppe nach Permutation
    res[i] <- mwRotTmp - mwGelbTmp ## berechne neue Differenz nach Permutation
  } 

  ## p-Wert: Wie wahrscheinlich ist das beobachtete Ergebnis (oder etwas noch extremeres), 
  ## falls die Nullhypothese (Behandlung hat keinerlei Effekt) stimmt ?
  p <- sum(res >= mDiff) / n ## Berechne simulierten p-Wert
  p ## Die Ausgabe der letzten Zeile einer Funktion ist automatisch der Rueckgabewert
} ## Um die Funktion in R zur Verfuegung zu stellen, muss man einmal die ganze Funktion markieren und ausfuehren

runSim(gL = gelbL, rL = rotL)

## Berechne p-Wert 100 mal (jedes mal mit anderen Permutationen)
nReps <- 100 ## Dauert auf meinem Laptop ca. 20 Sekunden
pVals <- rep(0, nReps)
for (i in 1:nReps) {
  pVals[i] <- runSim(gL = gelbL, rL = rotL, n = 10000)
}

pVals ## Viele Zahlen, daher unuebersichtlich
mean(pVals) ## Mittelwert; bei mir 0.0565
sd(pVals) ## Standardabweichung des geschaetzten p-Wertes; bei mir 0.002
boxplot(pVals) ## Boxplot zeichnen
hist(pVals) ## Histogramm zeichnen

## Wir sehen, dass der p-Wert mit 'nur' 10'000 Simulationen schon recht stabil ist